# START API API_NAMEViewSetDelete (Created by SPRINGLABS_DJANGO)
class API_NAMEViewSetDelete(mixins.DestroyModelMixin, viewsets.GenericViewSet):
    serializer_class = API_NAMESerializerDelete

    def destroy(self, request, *args, **kwargs):
        """
            Descripción de método destroy en documentación
        """
        instance = self.kwargs["pk"]
        serializer = self.get_serializer(instance, data=request.data)
        serializer.is_valid(raise_exception=True)
        result = serializer.destroy(instance, serializer.validated_data)
        response = response_maker_1(
            "Message", serializer.data, status.HTTP_200_OK)
        return Response(response)

API_NAMEDelete = API_NAMEViewSetDelete.as_view(
    name="API_NAME",
    description="Descripción de API_NAME",
    actions={
        'delete': 'destroy'
    }
)
# END API API_NAMEViewSetDelete (Created by SPRINGLABS_DJANGO)


import os
import json

from .primary_functions import (
    appendContent,
    replaceContent,
    findNReplace,
    findNDelete,
    deleteContent,
    notRecursiveModify,
    recursiveModify,
    getRangeStr,
    getRangeEnds,
    generateBackup,
    reverseChanges,
    modifyFilesNewVersion,
    createDirectoriesNewVersion,
    modifyFilesNewAPI,
    modifyFilesNewGraph,
    modifyFilesNewGroup,
    createDirectoriesNewGroup,
    deleteBackup
)
from . import PROJECT_PACKAGE, CONFIG_JSON, logging

global_error="Sorry, an internal error occurred"
def createVersion(version, project_name, old_versions):
    """
        Función encargada de crear nueva version en proyecto django

        Función encargada de crear nueva versión en proyecto django

        Parámetros:
            version         [String]    Nueva versión del proyecto
        Retorno:
            message,result  [Tuple]     Mensaje[String] y result[Boolean]
    """
    message, result = generateBackup(project_name=project_name)
    if result == False:
        message = "An internal error occurred while creating backup"
        logging.error(message)
        return global_error, False
    
    errors = []
    # Modifica archivos para nueva versión
    message, result = modifyFilesNewVersion(version=version,
        project_name=project_name,
        old_versions=old_versions)
    
    # Si falla al escribir archivos se agrega a arreglo
    if result == False:
        logging.error(message)
        errors.append([global_error, True])
    
    # Crea directorios y archivos dentro para nueva versión
    message, result = createDirectoriesNewVersion(version=version,
        project_name=project_name,
        old_versions=old_versions)
    
    # Si falla al modificar directorios se agrega a arreglo
    if result == False:
        logging.error(message)
        errors.append([global_error, True])
    try:
        with open(CONFIG_JSON) as file:
            data = json.load(file)
    except:
        message = f"An internal error occurred while creating backup {CONFIG_JSON}"
        logging.error(message)
        errors.append([global_error, True])
    else:
        objVersionDetail = {
            "version": version,
            "groups": ["users"],
            "graph_list":["ExampleDjangoUsersQueries"],
            "groups_detail": [
                {
                    "group": "users",
                    "apis": [],
                    "graph": [
                        {
                            "graph_name": "ExampleDjangoUsers",
                            "graph_type" : "open"
                        }
                    ]
                }
            ]
        }
        data["versions"].append(version)
        data['versions_detail'].append(objVersionDetail)
        try:
            with open(CONFIG_JSON, "w") as file:
                file.write(json.dumps(data))
        except:
            message = f"An internal error occurred while updating the file {CONFIG_JSON}"
            logging.error(message)
            errors.append([global_error, True])

    for error in errors:
        if error[1] == True:
            message, result = reverseChanges(project_name=project_name)
            os.chdir("..")
            os.remove(project_name + ".zip")
            os.chdir(project_name)
            return error[0], False
    deleteBackup(project_name=project_name)
    return "OK", True

def createAPI(version,group,name,http_verb,type_api,url,availability,project_name):
    """
        Función encargada de crear nueva api en proyecto django

        Función encargada de crear nueva api en proyecto django

        Parámetros:
            version         [String]    Versión donde se creará la API
            group           [String]    Grupo donde se creará la API
            name            [String]    Nombre de la API
            http_verb       [String]    HTTP Verb de la API (POST, PUT, DELETE)
            type_api        [String]    Tipo de la API (MODEL, OPEN)
            url             [String]    URL donde se podrá acceder a la API
            availability    [String]    Disponibilidad de la API (public, private)
            project_name    [String]    Nombre del proyecto
        Retorno:
            message,result  [Tuple]     Mensaje[String] y result[Boolean]
    """
    # Generar respaldo de proyecto
    message, result = generateBackup(project_name=project_name)
    if result == False:
        message = "An internal error occurred while creating backup"
        logging.error(message)
        return global_error, False

    errors = []

    # Modifica archivos para nueva api
    message, result = modifyFilesNewAPI(version=version,
        group=group,
        name=name,
        http_verb=http_verb,
        type_api=type_api,
        url=url,
        availability=availability,
        project_name=project_name)
    
    # Si falla al modificar directorios se agrega a arreglo
    if result == False:
        logging.error(message)
        errors.append([global_error, True])


    try:
        with open(CONFIG_JSON) as file:
            data = json.load(file)
    except:
        message = f"An internal error occurred while updating the file {CONFIG_JSON}"
        logging.error(message)
        errors.append([global_error, True])
    else:
        objApis = {
            "api_name": name,
            "http_verb": http_verb,
            "api_type" : type_api,
            "url": url,
            "availability" : availability
        }
        versions_detail = data['versions_detail']
        # Recorremos todas las versiones
        for index_versions, versions in enumerate(versions_detail):
            # Si la versión del ciclo es en donde vamos a insertar la api
            if versions['version'] == version:
                groups_detail = versions["groups_detail"]
                # Recorremos todos los grupos de esa versión
                for index_groups, group_detail in enumerate(groups_detail):
                    # Si el grupo del ciclo es en donde vamos a insertar la api
                    if group == group_detail['group']:
                        data['versions_detail'][index_versions]['groups_detail'][index_groups]['apis'].append(objApis)
        try:
            with open(CONFIG_JSON, "w") as file:
                file.write(json.dumps(data))
        except:
            message = f"An internal error occurred while updating the file {CONFIG_JSON}"
            logging.error(message)
            errors.append([global_error, True])
    # Si algo sale mal revertir los cambios y eliminar respaldo
    for error in errors:
        if error[1] == True:
            message, result = reverseChanges(project_name=project_name)
            os.chdir("..")
            os.remove(project_name + ".zip")
            os.chdir(project_name)
            return error[0], False
    deleteBackup(project_name=project_name)
    return "OK", True

def createGraph(version,group,name,type_graph,project_name):
    """
        Función encargada de crear nuevo graph en proyecto django

        Función encargada de crear nuevo graph en proyecto django

        Parámetros:
            version         [String]    Versión donde se creará el Graph
            group           [String]    Grupo donde se creará el Graph
            name            [String]    Nombre del Graph
            type_graph      [String]    Tipo del Graph (MODEL, OPEN)
            project_name    [String]    Nombre del proyecto
        Retorno:
            message,result  [Tuple]     Mensaje[String] y result[Boolean]
    """
    
    # Generar respaldo de proyecto
    message, result = generateBackup(project_name=project_name)
    if result == False:
        message = "An internal error occurred while creating backup"
        logging.error(message)
        return global_error, False

    errors = []

    # Modifica archivos para nuevo graph
    message, result = modifyFilesNewGraph(version=version,
        group=group,
        name=name,
        type_graph=type_graph,
        project_name=project_name)
    
    # Si falla al modificar directorios se agrega a arreglo
    if result == False:
        logging.error(message)
        errors.append([global_error, True])


    try:
        with open(CONFIG_JSON) as file:
            data = json.load(file)
    except:
        message = f"An internal error occurred while updating the file {CONFIG_JSON}"
        logging.error(message)
        errors.append([global_error, True])
    else:
        objGraph = {
            "graph_name": name,
            "graph_type": type_graph
        }
        versions_detail = data['versions_detail']
        # Recorremos todas las versiones
        for index_versions, versions in enumerate(versions_detail):
            # Si la versión del ciclo es en donde vamos a insertar el graph
            if versions['version'] == version:
                groups_detail = versions["groups_detail"]
                # Recorremos todos los grupos de esa versión
                for index_groups, group_detail in enumerate(groups_detail):
                    # Si el grupo del ciclo es en donde vamos a insertar el graph
                    if group == group_detail['group']:
                        data['versions_detail'][index_versions]['groups_detail'][index_groups]['graph'].append(objGraph)
        try:
            with open(CONFIG_JSON, "w") as file:
                file.write(json.dumps(data))
        except:
            message = f"An internal error occurred while updating the file {CONFIG_JSON}"
            logging.error(message)
            errors.append([global_error, True])
    # Si algo sale mal revertir los cambios y eliminar respaldo
    for error in errors:
        if error[1] == True:
            message, result = reverseChanges(project_name=project_name)
            os.chdir("..")
            os.remove(project_name + ".zip")
            os.chdir(project_name)
            return error[0], False
    deleteBackup(project_name=project_name)
    return "OK", True

def getVersions():
    """
        Función que obtiene las versiones existentes del proyecto.

        Esta función abre el archivo springlabs_json y obtiene las versiones
        que existen en el proyecto

        Retorno:
            version     [Array]       Lista de versiones existentes
        Exceptions:
            OSError                   Error al abrir el archivo json
    """
    try:
        with open(CONFIG_JSON, "r") as file:
            data = json.load(file)
        versions = data["versions"]
    except:
        message="An error occurred while opening file 'springlabs_django.json'"
        logging.error(message)
        versions=["1"]
    return versions

def createGroup(version, group, project_name):
    """
        Función encargada de crear nuevo grupo en proyecto django

        Parámetros:
            version         [String]    Versión donde se creará el grupo
            group           [String]    Grupo donde se creará el grupo
            project_name    [String]    Nombre del proyecto
        Retorno:
            message,result  [Tuple]     Mensaje[String] y result[Boolean]
        Excepciones:
            OSError                     Error al abrir archivos

    """

    # Generar respaldo de proyecto
    message, result = generateBackup(project_name=project_name)
    if result == False:
        message = "An internal error occurred while creating backup"
        logging.error(message)
        return global_error, False
    
    errors=[]
    message,result=modifyFilesNewGroup(version, group,project_name)
    if result == False:
        logging.error(message)
        errors.append([global_error, True])

    # Crea directorios y archivos dentro para un nuevo grupo
    message, result = createDirectoriesNewGroup(version, group, project_name)
    
    # Si falla al modificar directorios se agrega a arreglo
    if result == False:
        logging.error(message)
        errors.append([global_error, True])


    try:
        with open(CONFIG_JSON) as file:
            data = json.load(file)
    except Exception as e:
        message = f"An internal error occurred while updating the file {CONFIG_JSON}"
        logging.error(message)
        errors.append([global_error, True])
    else:
        groupcap=group.capitalize()
        changename=f"ExampleDjango{groupcap}"

        objGroupDetail = {
                    "group": group,
                    "apis": [],
                    "graph": [{"graph_name": changename,
                    "graph_type": "open"}]
                }

        for obj in data["versions_detail"]:
            if obj["version"]==version:
                obj["groups"].append(group)
                if not "groups_detail" in obj:
                    obj["groups_detail"]=[]
                obj["groups_detail"].append(objGroupDetail)
                if not "graph_list" in obj:
                    obj["graph_list"]=[]
                obj["graph_list"].append(f"{changename}Queries")
        try:
            with open(CONFIG_JSON, "w") as file:
                file.write(json.dumps(data))
        except:
            message = f"An internal error occurred while updating the file {CONFIG_JSON}"
            logging.error(message)
            errors.append([global_error, True])
    for error in errors:
        if error[1] == True:
            message, result = reverseChanges(project_name=project_name)
            os.chdir("..")
            os.remove(project_name + ".zip")
            os.chdir(project_name)
            return error[0], False
    deleteBackup(project_name=project_name)
    return "OK", True



# START API API_NAMEViewSetPost (Created by SPRINGLABS_DJANGO)
class API_NAMEViewSetPost(mixins.CreateModelMixin, viewsets.GenericViewSet):
    model = User
    queryset = User.objects.all()
    serializer_class = API_NAMESerializerPost
    
    def create(self, request, *args, **kwargs):
        """
            Descripción de método create en documentación
        """
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        response = response_maker_1(
            "Message", serializer.data, status.HTTP_200_OK)
        return Response(response)

API_NAMEPost = API_NAMEViewSetPost.as_view(
    name="API_NAME",
    description="Descripción de API_NAME",
    actions={
        'post': 'create'
    }
)
# END API API_NAMEViewSetPost (Created by SPRINGLABS_DJANGO)


# START API API_NAMEViewSetPut (Created by SPRINGLABS_DJANGO)
class API_NAMEViewSetPut(mixins.UpdateModelMixin, viewsets.GenericViewSet):
	serializer_class = API_NAMESerializerPut

	def update(self, request, *args, **kwargs):
		"""
			Descripción de método update en documentación
		"""
		instance = self.kwargs["pk"]
		serializer = self.get_serializer(instance, data=request.data)
		serializer.is_valid(raise_exception=True)
		self.perform_update(serializer)
		response=response_maker_1("Message",serializer.data,status.HTTP_200_OK)
		return Response(response)

API_NAMEPut = API_NAMEViewSetPut.as_view(
	name="API_NAME",
	description="Descripción de API_NAME",
	actions={
		'put': 'update'
	}
)
# END API API_NAMEViewSetPut (Created by SPRINGLABS_DJANGO)


import click
import os
import json
from . import PROJECT_PACKAGE, CONFIG_JSON, logging

global_error="Sorry, an internal error occurred"
try:
    with open(CONFIG_JSON) as file:
        data = json.load(file)
    versions = data["versions"]
    project_name = data["project_name"]
    versions_detail=data["versions_detail"]
except:
    message = f"The springlabs_django commands must be executed in the main project path ({CONFIG_JSON})"
    #logging.warning(message)
    raise click.BadParameter(message)

groups_api = ["options"]

def validate_manage():
    """
        Función que valida si se ejecuta comando en raiz de proyecto Django

        Función que valida si se ejecuta comando en la raiz del proyecto
        Django basandonos en el archivo manage.py
        Parámetros:
        Retorno:
            message,result      [Tuple] Mensaje de respuesta, Resultado booleano
        Excepciones:
            click.BadParameter  [Raise] Mensaje de parámetro incorrecto
    """
    list_dir = os.listdir()
    if not "manage.py" in list_dir:
        message = "The springlabs_django commands must be executed in the main project path (manage.py)"
        #logging.warning(message)
        return message, False
    return "OK", True


def validate_version_version(ctx, param, value):
    """
        Función que valida versión de subcomando create-version

        Función que valida si versión recibida es de tipo entero
        (ej. 1,2,3,4,...,n) y no existe ya una versión creada con ese
        número

        Parámetros:
            ctx         [Click Object]  Contexto de click
            param       [Click Option]  Parametro recibido
            value       [String]        Valor recibido como parámetro
        Retorno:
            value       [String]        Valor recibido como parámetro
        Excepciones:
            click.BadParameter  [Raise] Mensaje de parámetro incorrecto
    """
    message, result = validate_manage()
    if result == False:
        logging.warning(message)
        raise click.BadParameter(message)

    if not value.isdigit():
        message = f"Incorrect version '{value}'. It must be an integer value"
        logging.warning(message)
        raise click.BadParameter(message)
    if str(value) in versions:
        message = f"Duplicate version '{value}'. It must be a unique value"
        logging.warning(message)
        raise click.BadParameter(message)
    return value

def validate_api_name(ctx, param, value):
    """
        Función que valida que nombre de api este en formato correcto

        Función que valida que el nombre de la api este con formato correcto,
        es decir que solo sea alfanumérico

        Parámetros:
            ctx         [Click Object]  Contexto de click
            param       [Click Option]  Parametro recibido
            value       [String]        Valor recibido como parámetro
        Retorno:
            value       [String]        Valor recibido como parámetro
        Excepciones:
            click.BadParameter  [Raise] Mensaje de parámetro incorrecto
    """
    if value.isalnum():
        if not len(value) > 3 and len(value) < 16: 
            message = "The api name must contain between 4 and 15 characters (only letters and numbers)"
            logging.warning(message)
            raise click.BadParameter(message)
    else:
        message = "The group name must only contain letters and number (between 4 and 15 characters)"
        logging.warning(message)
        raise click.BadParameter(message)
    return value

def validate_api_version(ctx, param, value):
    """
        Función que valida versión de subcomando create-api

        Función que valida los grupos disponibles para generar una nueva
        api y llenar lista de choices en campo group de nueva api

        Parámetros:
            ctx         [Click Object]  Contexto de click
            param       [Click Option]  Parametro recibido
            value       [String]        Valor recibido como parámetro
        Retorno:
            value       [String]        Valor recibido como parámetro
        Excepciones:
            click.BadParameter  [Raise] Mensaje de parámetro incorrecto
    """
    try:
        with open(CONFIG_JSON) as file:
            data = json.load(file)
    except:
        message = f"An internal error occurred while reading file ({CONFIG_JSON})"
        logging.error(message)
        raise click.BadParameter(global_error)
    else:
        versions_detail = data['versions_detail']
        # Recorremos todas las versiones
        for versions in versions_detail:
            # Si la versión del ciclo es en donde vamos a insertar la api
            if versions['version'] == value:
                groups_api.remove("options")
                # Agregamos los grupos disponibles de esta version a la lista de grupos
                groups_api.extend(versions['groups'])
                return value
        message = "There are no groups created in this version"
        logging.warning(message)
        raise click.BadParameter(message)
    

def validate_api_name_verb(ctx, param, value):
    """
        Función que valida nombre de subcomando create-api

        Función que valida que nombre de la api no se encuentre ya registrado
        en la misma versión y mismo HTTP VERB de la aplicación.

        Parámetros:
            ctx         [Click Object]  Contexto de click
            param       [Click Option]  Parametro recibido
            value       [String]        Valor recibido como parámetro
        Retorno:
            value       [String]        Valor recibido como parámetro
        Excepciones:
            click.BadParameter  [Raise] Mensaje de parámetro incorrecto
    """
    if "version" in ctx.params and "name" in ctx.params and "group" in ctx.params:
        api_version = ctx.params['version']
        name_api = ctx.params['name']
        group_api = ctx.params['group']
    else:
        message = "Parameters are in the wrong order"
        logging.warning(message)
        raise click.BadParameter(message)
    try:
        with open(CONFIG_JSON) as file:
            data = json.load(file)
    except:
        message = f"An internal error occurred while reading file ({CONFIG_JSON})"
        logging.error(message)
        raise click.BadParameter(global_error)
    else:
        versions_detail = data['versions_detail']
        names_no_valid = []
        # Recorremos todas las versiones
        for versions in versions_detail:
            # Si la versión del ciclo es en donde vamos a insertar la api
            if versions['version'] == api_version:
                groups_detail = versions["groups_detail"]
                # Recorremos todos los grupos de esa versión
                for group_detail in groups_detail:
                    # Si el grupo del ciclo es en donde vamos a insertar la api
                    if group_api == group_detail['group']:
                        apis = group_detail['apis']
                        # Recorremos todas las apis de ese grupo
                        for api in apis:
                            verb = api["http_verb"]
                            name = api["api_name"]
                            name_verb = name.lower() + verb.upper()
                            # Agregamos a lista de names_no_valid los nombres de api no validos
                            names_no_valid.append(name_verb)
        find = name_api.lower() + value.upper()
        # Si el nombre de nuestra api ya se encuentra en el proyecto
        if find in names_no_valid:
            message = f"The api with name '{name_api}' and with HTTP '{value}' is already registered in the application"
            logging.warning(message)
            raise click.BadParameter(message)
        return value

def validate_api_url(ctx, param, value):
    """
        Función que valida url de subcomando create-api

        Función que valida que url de la api no se encuentre ya registrado
        en la misma versión.

        Parámetros:
            ctx         [Click Object]  Contexto de click
            param       [Click Option]  Parametro recibido
            value       [String]        Valor recibido como parámetro
        Retorno:
            value       [String]        Valor recibido como parámetro
        Excepciones:
            click.BadParameter  [Raise] Mensaje de parámetro incorrecto
    """
    if "version" in ctx.params:
        api_version = ctx.params['version']
    else:
        message = "Parameters are in the wrong order"
        logging.warning(message)
        raise click.BadParameter(message)
    # Validar que url no empiece en /
    if value.startswith("/") or value.endswith("/"):
        message = "The url cannot start or end with the character '/'"
        logging.warning(message)
        raise click.BadParameter(message)

    # Validar que parametros de url sean correctos
    urls = value.split("/")
    for url in urls:
        
        # Si empieza con parentesis se refiere a un argumento de la url
        if "(" in url:
            # Si NO empieza con (?P< y termina con ) es parámetro incorrecto
            if not url.startswith("(?P<") and url.endswith(")"):
                message = "Invalid url parameter"
                logging.warning(message)
                raise click.BadParameter(message)
            # Si la cantidad de parentesis de apertura '(' es diferente a los de cierre ')' es parámetro incorrecto
            if not url.count("(") == url.count(")"):
                message = "Invalid url parameter"
                logging.warning(message)
                raise click.BadParameter(message)
            # Si la cantidad de diplés de apertura '<' es diferente a los de cierre '>' es parámetro incorrecto
            if not url.count("<") == url.count(">"):
                message = "Invalid url parameter"
                logging.warning(message)
                raise click.BadParameter(message)
    try:
        with open(CONFIG_JSON) as file:
            data = json.load(file)
    except:
        message = f"An internal error occurred while reading file ({CONFIG_JSON})"
        logging.error(message)
        raise click.BadParameter(global_error)
    else:
        versions_detail = data['versions_detail']
        names_no_valid = []
        for versions in versions_detail:
            if versions['version'] == api_version:
                groups_detail = versions["groups_detail"]
                for group_detail in groups_detail:
                    apis = group_detail['apis']
                    for api in apis:
                        url = api["url"]
                        names_no_valid.append(url)
        if value in names_no_valid:
            message = f"The URL '{value}' is already registered in the project"
            logging.warning(message)
            raise click.BadParameter(message)
        return value

def validate_graph_name(ctx, param, value):
    """
        Función que valida que nombre de graph este en formato correcto

        Función que valida que el nombre de graph este con formato correcto,
        es decir que solo sea alfanumérico

        Parámetros:
            ctx         [Click Object]  Contexto de click
            param       [Click Option]  Parametro recibido
            value       [String]        Valor recibido como parámetro
        Retorno:
            value       [String]        Valor recibido como parámetro
        Excepciones:
            click.BadParameter  [Raise] Mensaje de parámetro incorrecto
    """
    if value.isalnum():
        if not len(value) > 3 and len(value) < 16: 
            message = "The graph name must contain between 4 and 15 characters (only letters and numbers)"
            logging.warning(message)
            raise click.BadParameter(message)
    else:
        message = "The graph name must only contain letters and number (between 4 and 15 characters)"
        logging.warning(message)
        raise click.BadParameter(message)
    if "version" in ctx.params and "group" in ctx.params:
        graph_version = ctx.params['version']
        group_graph = ctx.params['group']
    else:
        message = "Parameters are in the wrong order"
        logging.warning(message)
        raise click.BadParameter(message)
    name_graph = value
    try:
        with open(CONFIG_JSON) as file:
            data = json.load(file)
    except:
        message = f"An internal error occurred while reading file ({CONFIG_JSON})"
        logging.error(message)
        raise click.BadParameter(global_error)
    else:
        versions_detail = data['versions_detail']
        names_no_valid = []
        # Recorremos todas las versiones
        for versions in versions_detail:
            # Si la versión del ciclo es en donde vamos a insertar el graph
            if versions['version'] == graph_version:
                groups_detail = versions["groups_detail"]
                names_no_valid_version = versions["graph_list"]
                # Recorremos todos los grupos de esa versión
                for group_detail in groups_detail:
                    # Si el grupo del ciclo es en donde vamos a insertar el graph
                    if group_graph == group_detail['group']:
                        graphs = group_detail['graph']
                        # Recorremos todas las apis de ese grupo
                        for graph in graphs:
                            name = graph["graph_name"]
                            name_no_valid = name.lower()
                            # Agregamos a lista de names_no_valid los nombres de los graphs no validos
                            names_no_valid.append(name_no_valid)
        find = f"{name_graph.capitalize()}Queries"
        # Si el nombre de nuestro graph ya se encuentra en el proyecto
        if find in names_no_valid_version:
            message = f"A graph with name '{name_graph}' already exists in the project"
            logging.warning(message)
            raise click.BadParameter(message)
        return value
    return value

def validate_group_group(ctx, param, value):
    """
        Función que valida que nombre insertado para un nuevo grupo

        Función que valida que el nombre de graph este con formato correcto,
        es decir que solo sea alfanumérico

        Parámetros:
            ctx         [Click Object]  Contexto de click
            param       [Click Option]  Parametro recibido
            value       [String]        Valor recibido como parámetro
        Retorno:
            value       [String]        Valor recibido como parámetro
        Excepciones:
            click.BadParameter  [Raise] Mensaje de parámetro incorrecto
    """
    message, result = validate_manage()
    if result == False:
        #logging.warning(message)
        raise click.BadParameter(message)

    if value.isalnum():
        if not len(value) > 3 and len(value) < 16: 
            message = "The group name must contain between 4 and 15 characters (only letters and numbers)"
            logging.warning(message)
            raise click.BadParameter(message)
    else:
        message = "The group name must only contain letters and number (between 4 and 15 characters)"
        logging.warning(message)
        raise click.BadParameter(message)

    if "version" in ctx.params:
        insertversion=ctx.params["version"]
    else:
        message = "Parameters are in the wrong order"
        logging.warning(message)
        raise click.BadParameter(message)

    for detail in versions_detail:
        if detail["version"]==insertversion:
            if value.upper() in (gr.upper() for gr in detail["groups"]):
                message = f"group '{value}' already exists in version '{insertversion}'"
                logging.warning(message)
                raise click.BadParameter(message)
    return value


# Django libraries
# 3rd party libraries
from rest_framework import viewsets, mixins
from rest_framework.response import Response
from rest_framework import status
# Standard/core python libraries
# Our custom libraries (Managed by SPRINGLABS_DJANGO)
from core.utils import response_maker_1
from .serializers import *
from models_app.models import *



# Django libraries
from django.contrib.auth.models import User
from django.db.models import Q
# 3rd Party libraries
import graphene
from graphql_jwt.decorators import login_required, user_passes_test, staff_member_required

# Standard/core python libraries
# Our custom libraries
from models_app.models import UserApp
from .types import GRAPH_NAMEType

# EXAMPLE QUERY GRAPHENE (Managed by SPRINGLABS_DJANGO)
class GRAPH_NAMEQueries(object):
    # FIELDS
    examples = graphene.List(
        GRAPH_NAMEType,
        client=graphene.Argument(graphene.ID, required=True),
        role=graphene.Argument(graphene.String, required=False),
        agent=graphene.Argument(graphene.ID, required=False),
        description='Obtain all users.'
    )
    example = graphene.Field(
        GRAPH_NAMEType,
        id=graphene.Argument(graphene.ID, required=True),
        description='Lookup a user by ID or pk.'
    )

    # RESOLVES
    @login_required
    @user_passes_test(lambda user: user.is_staff or len(user.groups.filter(name='Administrador')) > 0)
    def resolve_examples(self, info, **kwargs):
        users = UserApp.objects.filter(client=kwargs['client'])
        result = []
        for user in users:
            objUser = {
                "id_user": user.user.pk,
                "username": user.user.username,
                "email": user.user.email,
                "first_name": user.user.first_name,
                "last_name": user.last_name
            }
            result.append(objUser)
        return result

    @login_required
    @user_passes_test(lambda user: user.is_staff or len(user.groups.filter(name='Administrador')) > 0)
    def resolve_example(self, info, **kwargs):
        if UserApp.objects.filter(pk=kwargs['id']).exists():
            objUserApp = UserApp.objects.get(pk=kwargs['id'])
            objUser = {
                "id_user": objUserApp.user.pk,
                "username": objUserApp.user.username,
                "email": objUserApp.user.email,
                "first_name": objUserApp.user.first_name,
                "last_name": objUserApp.last_name
            }
            return objUser
# END EXAMPLE QUERY GRAPHENE (Managed by SPRINGLABS_DJANGO)


import os
import re
from zipfile import ZipFile
import shutil
import pkgutil
import json
from . import CONFIG_JSON, PROJECT_PACKAGE


def appendContent(path, content, find=None, recursive=False):
    '''
    Agrega contenido al final del archivo especificado, si existe un valor en "find", agregará el contenido despues de la coincidencia.

    Atributos:
            path            [String]            Ruta de archivo a editar
            content         [String]            Nuevo contenido a agregar
            find            [String]            cadena a buscar, si es nulo el contenido se agregara al final
            recursive       [Boolean]           Indica si la operación será recursiva
    Retorno:
            result          [Tupla]             Resultado de operación
    Excepciones:
            OSError                             Error al crear el archivo
            OSError                             Error al escribir el contenido
    '''
    if not os.path.exists(path):
        return f'File {path} does not exist', False
    if find == None:
        try:
            """ if not os.path.exists(path):
                    open(path, "x")
            with open(f'{path}.bak', "w") as f:
                    f.write(open(path).read()) """
            with open(path, "a") as f:
                newContent = f'\n{content}'
                f.write(newContent)
            return f'The file "{path}" has been updated successfully.', True
        except Exception as exc:
            return f'Error writing content:{exc}', False
    else:
        if recursive == False:
            return notRecursiveModify(path, content, find, 'append')
        elif recursive == True:
            return recursiveModify(path, content, find, 'append')

        return f'Invalid value for recursive parameter', False


def replaceContent(path, content, start, end='--end', recursive=False):
    '''
    Remplaza cierto contenido del archivo especificado.

    Atributos:
            path            [String]            Ruta de archivo a editar
            content         [String]            Nuevo contenido a agregar
            start           [String]            Parametro a partir del cual se iniciará el remplazo hasta encontrar el parametro "end"
            recursive       [Boolean]           Indica si la operación será recursiva
    Retorno:
            result          [Tupla]             Resultado de operación
    Excepciones:
            OSError                             Error al escribir el contenido
    '''
    if not os.path.exists(path):
        return f'File {path} does not exists', False

    starts = getRangeStr(path, start)
    ends = getRangeEnds(path, end)
    if ends == None or len(ends) == 0:
        return f'The file does not have end delimiters: "{end}"', False
    if starts == None or len(starts) == 0:
        return f'The file does not have start delimiters: "{start}"', False
    textsReplace = []
    for idx, _ in enumerate(ends):
        if len(starts) <= idx or starts[idx][1] > ends[idx][0]:
            break
        textsReplace.append([starts[idx][1] + 1,  ends[idx][0] - 1])
    file = open(path).read()
    if len(textsReplace) == 0:
        return f'The file has wrong start delimiters "{start}" and/or end delimiters "{end}", please check', False
    if recursive == False:
        return notRecursiveModify(path, content, file[textsReplace[0][0]:textsReplace[0][1]], 'replace')
    elif recursive == True:
        try:
            for text in textsReplace:
                recursiveModify(
                    path, content, file[text[0]:text[1]], 'replace')
            return f'File "{path}" was successfully updated', True
        except Exception as exc:
            return f'Error writing content: {exc}', False

    return f'Invalid value for recursive parameter', False


def findNReplace(path, content, find, recursive=False):
    '''
    Remplaza cierto contenido del archivo especificado.

    Atributos:
            path            [String]            Ruta de archivo a editar
            content         [String]            Nuevo contenido a agregar
            find            [String]            Se hará un "search & replace de la valor dado
            recursive       [Boolean]           Indica si la operación será recursiva
    Retorno:
            result          [Tupla]             Resultado de operación
    Excepciones:
            OSError                             Error al escribir el contenido
    '''
    if not os.path.exists(path):
        return f'File {path} does not exists', False
    if recursive == False:
        return notRecursiveModify(path, content, find, 'replace')
    elif recursive == True:
        return recursiveModify(path, content, find, 'replace')

    return f'Invalid value for recursive parameter', False


def findNDelete(path, find, recursive=False):
    '''
    Elimina cierto contenido del archivo especificado.

    Atributos:
            path            [String]            Ruta de archivo a editar
            find            [String]            Se hará un "search & replace de la valor dado
            recursive       [Boolean]           Indica si la operación será recursiva
    Retorno:
            result          [Tupla]             Resultado de operación
    Excepciones:
            OSError                             Error al escribir el contenido
    '''
    if not os.path.exists(path):
        return f'File {path} does not exists', False
    if recursive == False:
        return findNReplace(path, '', find)
    elif recursive == True:
        return findNReplace(path, '', find, recursive=True)

    return f'Invalid value for recursive parameter', False


def deleteContent(path, start, end='--end', recursive=False):
    '''
    Elimina cierto contenido del archivo especificado.

    Atributos:
            path            [String]            Ruta de archivo a editar
            start           [String]            Parametro a partir del cual se iniciará el remplazo hasta encontrar el parametro "end"
            recursive       [Boolean]           Indica si la operación será recursiva
    Retorno:
            result          [Tupla]             Resultado de operación
    Excepciones:
            OSError                             Error al escribir el contenido
    '''
    if not os.path.exists(path):
        return f'The file does not exists', False
    if recursive == False:
        return replaceContent(path, '', start, end)
    elif recursive == True:
        return replaceContent(path, '', start, end, recursive=True)

    return f'Invalid value for recursive parameter', False


def notRecursiveModify(path, content, find, type):
    '''
    Modifica un archivo dado, en la coincidencia "find" con el contenido de "content" (solo primera coincidencia).

    Atributos:
            path            [String]            Ruta de archivo a editar
            content         [String]            Nuevo contenido a agregar
            find            [String]            Se hará un "search & replace de la valor dado
            type	        [String]            Indica el tipo de operación
    Retorno:
            result          [Tupla]             Resultado de operación
    Excepciones:
            OSError                             Error al escribir el contenido
    '''
    file = open(path).read()
    matchFound = False
    find = find.replace("(", r'\(')
    find = find.replace(")", r'\)')
    find = find.replace("{", r'\{')
    find = find.replace("}", r'\}')
    find = find.replace("[", r'\[')
    find = find.replace("]", r'\]')
    find = find.replace("+", r'\+')
    find = find.replace("<", r'\<')
    find = find.replace("-", r'\-')
    find = find.replace("!", r'\!')
    find = find.replace("*", r'\*')
    find = find.replace("~", r'\~')
    find = find.replace("¬", r'\¬')
    find = find.replace(",", r'\,')
    find = find.replace(":", r'\:')
    find = find.replace(".", r'\.')

    matches = re.finditer(find, file)
    for m in matches:
        matchFound = True
        break
    if matchFound == False:
        return f'No matches found with "{find}"', False
    if type == 'append':
        newContent = f'{file[: m.end()]}  \n{content} \n{file[m.end() + 1:]}'
    elif type == 'replace':
        newContent = f'{file[: m.start()]}{content}{file[m.end():]}'
    else:
        return f'Invalid replacement type', False
    try:
        '''with open(f'{path}.bak', "w") as f:
                f.write(open(path).read())'''
        with open(path, "w") as f:
            f.write(newContent)
        return f'File "{path}" was successfully updated', True
    except Exception as exc:
        return f'Error writing content: {exc}', False


def recursiveModify(path, content, find, type):
    '''
    Modifica un archivo dado, en la coincidencia "find" con el contenido de "content" de manera recursiva.

    Atributos:
            path            [String]            Ruta de archivo a editar
            content         [String]            Nuevo contenido a agregar
            find            [String]            Se hará un "search & replace de la valor dado
            type	        [String]            Indica el tipo de operación
    Retorno:
            result          [Tupla]             Resultado de operación
    Excepciones:
            OSError                             Error al escribir el contenido
    '''
    if type == 'append':
        newContent = f'{find} \n{content}\n'
    elif type == 'replace':
        newContent = content
    else:
        return f'Invalid replacement type', False
    try:
        file = open(path).read()
        '''with open(f'{path}.bak', "w") as f:
			f.write(open(path).read())'''
        with open(path, "w") as f:
            f.write(file.replace(find, newContent))
        return f'File "{path}" was successfully updated', True
    except Exception as exc:
        return f'Error writing content: {exc}', False


def getRangeStr(path, find):
    '''
    Obtiene los indices del primer y ultimo caracter del valor de "find".

    Atributos:
            path            [String]            Ruta de archivo a editar
            find            [String]            Se hará un search de la valor dado
    Retorno:
            arr          	[Lista]            Indices
    Excepciones:
            OSError                             Error al escribir el contenido
    '''
    arr = []
    file = open(path).read()
    find = find.replace("(", r'\(')
    find = find.replace(")", r'\)')
    find = find.replace("{", r'\{')
    find = find.replace("}", r'\}')
    find = find.replace("[", r'\[')
    find = find.replace("]", r'\]')
    find = find.replace("+", r'\+')
    find = find.replace("<", r'\<')
    find = find.replace("-", r'\-')
    find = find.replace("!", r'\!')
    find = find.replace("*", r'\*')
    find = find.replace("~", r'\~')
    find = find.replace("¬", r'\¬')
    find = find.replace(",", r'\,')
    find = find.replace(":", r'\:')
    find = find.replace(".", r'\.')

    matches = re.finditer(find, file)
    for m in matches:
        arr.append([m.start(), m.end()])
    return arr


def getRangeEnds(path, find):
    '''
    Obtiene los indices del primer y ultimo caracter de la linea donde se encuentre el valor de "find".

    Atributos:
            path            [String]            Ruta de archivo a editar
            find            [String]            Se hará un "search & replace de la valor dado
    Retorno:
            arr          	[Lista]            Indices
    Excepciones:
            OSError                             Error al escribir el contenido
    '''
    with open(path) as file:
        for line in file:
            if find in line:
                return getRangeStr(path, line)


def generateBackup(project_name, path=os.getcwd()):
    """
        Función que genera un respaldo del proyecto 

        Función encargada de generar un respaldo del proyecto antes de realizar alguna
        acción, se genera un zip con el nombre del proyecto por si algo llega a fallar
        revertir los cambios 
        
        Parametros:
            project_name (String): Nombre del proyecto a generar respaldo
            path (String): Ruta donde se encuentra el proyecto a respaldar 
        Retorno:
            message,result (Tuple): Mensaje[String] y result[Boolean]

    """
    oldPath = path
    os.chdir("..")
    shutil.make_archive(project_name, 'zip', base_dir=project_name)
    os.chdir(oldPath)
    return "OK", True
    

def reverseChanges(project_name, path=os.getcwd()):
    """
        Función que revierte los cambios de una acción 

        Función encargada de revertir los cambios de una acción, hace que se haga un unzip
        del respaldo realizado antes de la acción y poder eliminar el contenido con errores 
        
        Parametros:
            project_name (String): Nombre del proyecto a revertir cambios
            path (String): Ruta donde se encuentra el proyecto a revertir 
        Retorno:
            message,result (Tuple): Mensaje[String] y result[Boolean]

    """
    try:
        oldPath = path
        os.chdir("..")
        shutil.rmtree(project_name)
        botZip = f'{project_name}.zip'
        with ZipFile(botZip, 'r') as zip_ref:
            zip_ref.extractall(os.getcwd())
        os.chdir(oldPath)
        return "OK", True
    except Exception as e:
        return str(e), False


def deleteBackup(project_name, path=os.getcwd()):
    """
        Función que elimina el respaldo de un proyecto 

        Función encargada de eliminar el respaldo de un proyecto al terminar una acción 
        
        Parametros:
            project_name (String): Nombre del proyecto que se desea eliminar respaldo
            path (String): Ruta donde se encuentra el proyecto que se desea eliminar respaldo
        Retorno:
            message,result (Tuple): Mensaje[String] y result[Boolean]

    """
    try:
        oldPath = os.getcwd()
        os.chdir("..")
        os.remove(project_name + ".zip")
        os.chdir(oldPath)
        return "OK", True
    except Exception as e:
        return str(e), False

def modifyFilesNewVersion(version, project_name, old_versions):
    """
        Función que modifica archivos para agregar nueva versión

        Función que modifica los archivos de Django en los que se tiene
        que agregar la nueva versión (No contempla nuevas carpetas)

    Args:
        version (String): Nueva versión del proyecto
        project_name (String): Nombre del proyecto
        old_versions (List): Lista de versiones existentes en proyecto
    Returns:
        message,result (Tuple): Mensaje[String] y result[Boolean]
    Exceptions:
        exception ([type]): [description]
    """

    errors = []
    # Crear version en project_name/urls.py
    # Importar public versions de nueva versión
    path = f"{project_name}/urls.py"
    find = "# PUBLIC VERSIONS URLS (Managed by SPRINGLABS_DJANGO)"
    content = f"from api.v{version}.public_urls import public_urls as public_urls_v{version}"
    message, result = appendContent(path=path,
        content=content,
        find=find)
    if result == False:
        errors.append([message, True])

    # Agregar nueva version a urls_versions
    find = "urls_versions ="
    for index, old_version in enumerate(old_versions):
        if index == 0:
            public_urls = f" public_urls_v{old_version}"
        else:
            public_urls = f" + public_urls_v{old_version}"
        find = find + public_urls
    content = find + f" + public_urls_v{version}"

    message, result = findNReplace(path=path,
        content=content,
        find=find)
    if result == False:
        errors.append([message, True])
    
    # Crear version en api/urls.py
    # Agregar nueva version a url_patterns
    path = "api/urls.py"
    find = "# VERSIONS URLS (Managed by SPRINGLABS_DJANGO)"
    content = f"""
    url(r'^v{version}/', include('api.v{version}.urls', namespace='v{version}')),"""
    message, result = appendContent(path=path,
        content=content,
        find=find)
    if result == False:
        errors.append([message, True])
    
    # Crear version en core/documentation.py
    # Crear variables urls de version
    path = "core/documentation.py"
    find = "# DOC STRING VERSIONS (Managed by SPRINGLABS_DJANGO)"
    content = f"""
# VERSION {version}.0
# PUBLIC
title_v{version}_0 = "{project_name} v{version}.0"
description_v{version}_0 = "Descripción del proyecto (v{version})"
namespace_v{version}_0 = "api-docs-v{version}-0"
urls_v{version}_0 = "api.v{version}.urls"
# PRIVATE
title_v{version}_0_private = "{project_name} (Private) v{version}.0"
description_v{version}_0_private = "Descripción del proyecto (Private) (v{version})"
namespace_v{version}_0_private = "api-docs-v{version}-0-private"
urls_v{version}_0_private = "api.v{version}.urls"
# END VERSION {version}.0"""
    message, result = appendContent(path=path,
        content=content,
        find=find)
    
    if result == False:
        errors.append([message, True])
    
    # Crear version en graphql_api/urls.py
    # Agregar nueva version a url_patterns
    path = "graphql_api/urls.py"
    find = "# VERSIONS URLS (Managed by SPRINGLABS_DJANGO)"
    content = f"""
    url(r'^v{version}/', include('graphql_api.v{version}.urls', namespace='v{version}')),"""
    message, result = appendContent(path=path,
        content=content,
        find=find)
    if result == False:
        errors.append([message, True])
    

    for error in errors:
        if error[1] == True:
            return error[0], False
    
    return "OK", True

def createDirectoriesNewVersion(version, project_name, old_versions):
    """
        Función que modifica directorios para agregar nueva versión

        Función que modifica los directorios de Django y sus archivos dentro 
        en los que se tiene que agregar la nueva versión

    Args:
        version (String): Nueva versión del proyecto
        project_name (String): Nombre del proyecto
        old_versions (List): Lista de versiones existentes en proyecto
    Returns:
        message,result (Tuple): Mensaje[String] y result[Boolean]
    Exceptions:
        exception ([type]): [description]
    """

    errors = []
    
    # Crear carpeta API nueva versión con archivos
    os.chdir("api")
    new_version = f"v{version}"
    os.mkdir(new_version)
    base_urls_py = "/core/new_version/api/baseUrls.py"
    base_public_urls_py = "/core/new_version/api/basePublicUrls.py"
    base_private_urls_py = "/core/new_version/api/basePrivateUrls.py"
    base_serializers_py = "/core/new_version/api/baseSerializers.py"
    base_views_py = "/core/new_version/api/baseViews.py"

    # Obtiene contenido de core/new_version/api/baseUrls.py
    try:
        template_urls_py = pkgutil.get_data(PROJECT_PACKAGE, base_urls_py).decode('utf-8')
        template_urls_py = template_urls_py.replace("name_version", new_version)
    except:
        message = f"An internal error occurred in core/{new_version}/api/baseUrls.py"
        errors.append([message, True])

    # Obtiene contenido de core/new_version/api/basePublicUrls.py
    try:
        template_public_urls_py = pkgutil.get_data(PROJECT_PACKAGE, base_public_urls_py).decode('utf-8')
        template_public_urls_py = template_public_urls_py.replace("name_version", new_version)
    except:
        message = f"An internal error occurred in core/{new_version}/api/basePublicUrls.py"
        errors.append([message, True])

    # Obtiene contenido de core/new_version/api/basePrivateUrls.py
    try:
        template_private_urls_py = pkgutil.get_data(PROJECT_PACKAGE, base_private_urls_py).decode('utf-8')
        template_private_urls_py = template_private_urls_py.replace("name_version", new_version)
    except:
        message = f"An internal error occurred in core/{new_version}/api/basePrivateUrls.py"
        errors.append([message, True])

    # Obtiene contenido de core/new_version/api/baseSerializers.py
    try:
        template_serializers_py = pkgutil.get_data(PROJECT_PACKAGE, base_serializers_py).decode('utf-8')
    except:
        message = f"An internal error occurred in core/{new_version}/api/baseSerializers.py"
        errors.append([message, True])

    # Obtiene contenido de core/new_version/api/baseViews.py
    try:
        template_views_py = pkgutil.get_data(PROJECT_PACKAGE, base_views_py).decode('utf-8')
    except:
        message = f"An internal error occurred in core/{new_version}/api/baseViews.py"
        errors.append([message, True])

    # Escribe contenido base en archivo api/new_version/urls.py
    with open(f"{new_version}/urls.py", "w") as file:
        file. write(template_urls_py)
    
    # Escribe contenido base en archivo api/new_version/public_urls.py
    with open(f"{new_version}/public_urls.py", "w") as file:
        file. write(template_public_urls_py)
    
    # Escribe contenido base en archivo api/new_version/private_urls.py
    with open(f"{new_version}/private_urls.py", "w") as file:
        file. write(template_private_urls_py)
    os.mkdir(f"{new_version}/users")
    
    # Escribe contenido base en archivo api/new_version/users/serializers.py
    with open(f"{new_version}/users/serializers.py", "w") as file:
        file. write(template_serializers_py)
    
    # Escribe contenido base en archivo api/new_version/users/views.py
    with open(f"{new_version}/users/views.py", "w") as file:
        file. write(template_views_py)
    os.chdir("..")
    
    # Crear carpeta GRAPHQL_API nueva versión con archivos
    os.chdir("graphql_api")
    new_version = f"v{version}"
    os.mkdir(new_version)
    base_urls_py = "/core/new_version/graphql_api/baseUrls.py"
    base_api_py = "/core/new_version/graphql_api/baseApi.py"
    base_schema_py = "/core/new_version/graphql_api/baseSchema.py"
    base_types_py = "/core/new_version/graphql_api/baseTypes.py"

    # Obtiene contenido de core/new_version/graphql_api/baseUrls.py
    try:
        template_urls_py = pkgutil.get_data(PROJECT_PACKAGE, base_urls_py).decode('utf-8')
        template_urls_py = template_urls_py.replace("name_version", new_version)
    except:
        message = f"An internal error occurred in core/{new_version}/graphql_api/baseUrls.py"
        errors.append([message, True])

    # Obtiene contenido de core/new_version/graphql_api/baseApi.py
    try:
        template_api_py = pkgutil.get_data(PROJECT_PACKAGE, base_api_py).decode('utf-8')
    except:
        message = f"An internal error occurred in core/{new_version}/graphql_api/baseApi.py"
        errors.append([message, True])


    # Obtiene contenido de core/new_version/graphql_api/baseSchema.py
    try:
        template_schema_py = pkgutil.get_data(PROJECT_PACKAGE, base_schema_py).decode('utf-8')
    except:
        message = f"An internal error occurred in core/{new_version}/graphql_api/baseSchema.py"
        errors.append([message, True])

    # Obtiene contenido de core/new_version/graphql_api/baseTypes.py
    try:
        template_types_py = pkgutil.get_data(PROJECT_PACKAGE, base_types_py).decode('utf-8')
    except:
        message = f"An internal error occurred in core/{new_version}/graphql_api/baseTypes.py"
        errors.append([message, True])

    # Escribe contenido base en archivo graphql_api/new_version/urls.py
    with open(f"{new_version}/urls.py", "w") as file:
        file. write(template_urls_py)
    
    # Escribe contenido base en archivo graphql_api/new_version/api.py
    with open(f"{new_version}/api.py", "w") as file:
        file. write(template_api_py)
    os.mkdir(f"{new_version}/users")
    
    # Escribe contenido base en archivo graphql_api/new_version/users/schema.py
    with open(f"{new_version}/users/schema.py", "w") as file:
        file. write(template_schema_py)
    
    # Escribe contenido base en archivo graphql_api/new_version/users/types.py
    with open(f"{new_version}/users/types.py", "w") as file:
        file. write(template_types_py)
    os.chdir("..")

    # Crear carpeta TESTS nueva versión con archivos
    os.chdir("tests")
    new_version = f"v{version}"
    os.mkdir(new_version)
    os.mkdir(f"{new_version}/integration_tests")
    os.mkdir(f"{new_version}/integration_tests/utils")
    os.mkdir(f"{new_version}/unit_tests")


    base_test_apis_py = "/core/new_version/tests/baseTestApis.py"
    base_test_functions1_py = "/core/new_version/tests/baseTestFunctions1.py"
    base_test_serializers_py = "/core/new_version/tests/baseTestSerializers.py"
    base_test_functions_py = "/core/new_version/tests/baseTestFunctions.py"

    # Obtiene contenido de core/new_version/tests/baseTestApis.py
    try:
        template_test_apis_py = pkgutil.get_data(PROJECT_PACKAGE, base_test_apis_py).decode('utf-8')
    except:
        message = f"An internal error occurred in core/{new_version}/tests/baseTestApis.py"
        errors.append([message, True])
    
    # Obtiene contenido de core/new_version/tests/baseTestFunctions1.py
    try:
        template_test_functions1_py = pkgutil.get_data(PROJECT_PACKAGE, base_test_functions1_py).decode('utf-8')
    except:
        message = f"An internal error occurred in core/{new_version}/tests/baseTestFunctions1.py"
        errors.append([message, True])
    
    # Obtiene contenido de core/new_version/tests/baseTestSerializers.py
    try:
        template_test_serializers_py = pkgutil.get_data(PROJECT_PACKAGE, base_test_serializers_py).decode('utf-8')
    except:
        message = f"An internal error occurred in core/{new_version}/tests/baseTestSerializers.py"
        errors.append([message, True])
    
    # Obtiene contenido de core/new_version/tests/baseTestFunctions.py
    try:
        template_test_functions_py = pkgutil.get_data(PROJECT_PACKAGE, base_test_functions_py).decode('utf-8')
    except:
        message = f"An internal error occurred in core/{new_version}/tests/baseTestFunctions.py"
        errors.append([message, True])
    

    # Escribe contenido base en archivo tests/new_version/integration_tests/test_apis.py
    with open(f"{new_version}/integration_tests/test_apis.py", "w") as file:
        file. write(template_test_apis_py)
    
    # Escribe contenido base en archivo tests/new_version/integration_tests/test_functions1.py
    with open(f"{new_version}/integration_tests/test_functions1.py", "w") as file:
        file. write(template_test_functions1_py)
    
    # Escribe contenido base en archivo tests/new_version/integration_tests/test_serializers.py
    with open(f"{new_version}/integration_tests/test_serializers.py", "w") as file:
        file. write(template_test_serializers_py)
    
    # Escribe contenido base en archivo tests/new_version/unit_tests/test_functions.py
    with open(f"{new_version}/unit_tests/test_functions.py", "w") as file:
        file. write(template_test_functions_py)
    os.chdir("..")
    
    for error in errors:
        if error[1] == True:
            return error[0], False
    
    return "OK", True

def modifyFilesNewAPI(version,group,name,http_verb,type_api,url,availability,project_name):
    """
        Función encargada de modificar archivos para nueva api en django

        Función encargada de modificar los archivos necesarios para crear una nueva api
        en el proyecto django.

        Parámetros:
            version         [String]    Versión donde se creará la API
            group           [String]    Grupo donde se creará la API
            name            [String]    Nombre de la API
            http_verb       [String]    HTTP Verb de la API (POST, PUT, DELETE)
            type_api        [String]    Tipo de la API (MODEL, OPEN)
            url             [String]    URL donde se podrá acceder a la API
            availability    [String]    Disponibilidad de la API (public, private)
            project_name    [String]    Nombre del proyecto
        Retorno:
            message,result  [Tuple]     Mensaje[String] y result[Boolean]
    """
    errors = []
    
    # Obtener variables dependientes del HTTP_VERB
    if http_verb == "POST":
        base_views = "post_views_"
        base_serializers = "post_serializers_"
    elif http_verb == "PUT":
        base_views = "put_views_"
        base_serializers = "put_serializers_"
    elif http_verb == "DELETE":
        base_views = "delete_views_"
        base_serializers = "delete_serializers_"
    
    # Obtener variables dependientes del TYPE_API
    if type_api=="open":
        base_views = base_views+"open.py"
        base_serializers = base_serializers+"open.py"
    elif type_api=="model":
        base_views = base_views+"model.py"
        base_serializers = base_serializers+"model.py"
    
    # Obtener variables dependientes del AVAILABILITY
    if availability=="public":
        general_urls = "public_urls.py"
    elif availability=="private":
        general_urls = "private_urls.py"
    
    base_views_py = f"/core/new_api/api/{base_views}"
    base_serializers_py = f"/core/new_api/api/{base_serializers}"
    # Obtiene contenido de /core/new_api/api/{base_views}
    try:
        template_views_py = pkgutil.get_data(PROJECT_PACKAGE, base_views_py).decode('utf-8')
        template_views_py = template_views_py.replace("API_NAME", name.capitalize())
    except:
        message = f"An internal error occurred in /core/new_api/api/{base_views}"
        errors.append([message, True])
    
    # Obtiene contenido de /core/new_api/api/{base_serializers}
    try:
        template_serializers_py = pkgutil.get_data(PROJECT_PACKAGE, base_serializers_py).decode('utf-8')
        template_serializers_py = template_serializers_py.replace("API_NAME", name.capitalize())
    except:
        message = f"An internal error occurred in /core/new_api/api/{base_serializers}"
        errors.append([message, True])
    
    # Escribe en archivo views.py
    path = f"api/v{version}/{group}/views.py"
    message, result = appendContent(path=path,
        content=template_views_py)
    if result == False:
        errors.append([message, True])

    # Escribe en archivo serializers.py
    path = f"api/v{version}/{group}/serializers.py"
    message, result = appendContent(path=path,
        content=template_serializers_py)
    if result == False:
        errors.append([message, True])

    # Escribe en urls public/private
    views_name = f"{name.capitalize()}{http_verb.capitalize()}"
    path = f"api/v{version}/{general_urls}"
    find = f"# {group.upper()} URLS (Managed by SPRINGLABS_DJANGO)"
    content = f"    url(r'^{url}/$', {group.lower()}_views.{views_name}),"
    message, result = appendContent(path=path,
        content=content,
        find=find)
    if result == False:
        errors.append([message, True])

    for error in errors:
        if error[1] == True:
            return error[0], False

    return "OK", True

def modifyFilesNewGraph(version,group,name,type_graph,project_name):
    """
        Función encargada de modificar archivos para nuevo graph en django

        Función encargada de modificar los archivos necesarios para crear un nuev graph
        en el proyecto django.

        Parámetros:
            version         [String]    Versión donde se creará el Graph
            group           [String]    Grupo donde se creará el Graph
            name            [String]    Nombre del Graph
            type_graph      [String]    Tipo del Graph (MODEL, OPEN)
            project_name    [String]    Nombre del proyecto
        Retorno:
            message,result  [Tuple]     Mensaje[String] y result[Boolean]
    """
    errors = []
    

    graph_version = version
    group_graph = group
    try:
        with open(CONFIG_JSON) as file:
            data = json.load(file)
    except:
        message = f"An internal error occurred while reading file ({CONFIG_JSON})"
        return message, False
    else:
        versions_detail = data['versions_detail']
        old_graphs = []
        old_graphs_version = []
        index_version_modify = 0
        # Recorremos todas las versiones
        for index, versions in enumerate(versions_detail):
            # Si la versión del ciclo es en donde vamos a insertar el graph
            if versions['version'] == graph_version:
                groups_detail = versions["groups_detail"]
                index_version_modify = index
                old_graphs_version = list(versions['graph_list'])
                # Recorremos todos los grupos de esa versión
                for group_detail in groups_detail:
                    # Si el grupo del ciclo es en donde vamos a insertar el graph
                    if group_graph == group_detail['group']:
                        graphs = group_detail['graph']
                        # Recorremos todas las apis de ese grupo
                        for graph in graphs:
                            old_name = graph["graph_name"]
                            # Agregamos a lista de old_graphs los nombres de los graphs existentes
                            old_graphs.append(old_name)
    
    example_django_name = f"ExampleDjango{group.capitalize()}"
    # Si se encuentra ExampleDjango en los graphs existentes
    if example_django_name in old_graphs:
        delete_example = True 
    else:
        delete_example = False
    # Obtener variables dependientes del TYPE_GRAPH
    base_schema = "schema.py"
    if type_graph=="open":
        base_types = "types_open.py"
    elif type_graph=="model":
        base_types = "types_model.py"
    
    base_schema_py = f"/core/new_graph/graph/{base_schema}"
    base_types_py = f"/core/new_graph/graph/{base_types}"

    # Obtiene contenido de /core/new_graph/graph/{base_schema}
    try:
        template_schema_py = pkgutil.get_data(PROJECT_PACKAGE, base_schema_py).decode('utf-8')
        template_schema_py = template_schema_py.replace("GRAPH_NAME", name.capitalize())
    except:
        message = f"An internal error occurred in /core/new_graph/graph/{base_schema}"
        errors.append([message, True])
    
    # Obtiene contenido de /core/new_graph/graph/{base_types}
    try:
        template_types_py = pkgutil.get_data(PROJECT_PACKAGE, base_types_py).decode('utf-8')
        template_types_py = template_types_py.replace("GRAPH_NAME", name.capitalize())
    except:
        message = f"An internal error occurred in /core/new_graph/graph/{base_types}"
        errors.append([message, True])
    
    # Obtiene texto a buscar de import en schema.py y api.py
    find_import_schema = "from .types import"
    find_import_api = f"from .{group}.schema import"
    find_query_api = f"class Query("
    # Arma FIND de class Query api.py
    for index, old_graph in enumerate(old_graphs_version):
        old_graph_name = old_graph
        if index == 0:
            find_query_api = find_query_api+f"{old_graph_name}"
        else:
            find_query_api = find_query_api+f", {old_graph_name}"
    base_new_query_api = str(find_query_api)
    find_query_api = find_query_api + ", graphene.ObjectType):"
    # Arma find de import schema.py e import api.py 
    for index, old_graph in enumerate(old_graphs):
        if old_graph != example_django_name:
            old_graph_name = old_graph.capitalize()
        else:
            old_graph_name = old_graph
        if index == 0:
            
            find_import_schema = find_import_schema+f" {old_graph_name}Type"
            find_import_api = find_import_api+f" {old_graph_name}Queries"
        else:
            find_import_schema = find_import_schema+f", {old_graph_name}Type"
            find_import_api = find_import_api+f", {old_graph_name}Queries"
    
    # Elimina ExampleDjango de archivo schema.py y types.py
    if delete_example==True:
        # Elimina ExampleDjango de archivo schema.py
        path = f"graphql_api/v{version}/{group}/schema.py"
        message, result = deleteContent(path=path,
            start="# EXAMPLE QUERY GRAPHENE (Managed by SPRINGLABS_DJANGO)",
            end="# END EXAMPLE QUERY GRAPHENE (Managed by SPRINGLABS_DJANGO)")
        if result == False:
            errors.append([message, True])
        
        message, result = findNDelete(path=path,
            find="# EXAMPLE QUERY GRAPHENE (Managed by SPRINGLABS_DJANGO)")
        if result == False:
            errors.append([message, True])
        
        message, result = findNDelete(path=path,
            find="# END EXAMPLE QUERY GRAPHENE (Managed by SPRINGLABS_DJANGO)")
        if result == False:
            errors.append([message, True])
        
        # Elimina ExampleDjango de archivo types.py
        path = f"graphql_api/v{version}/{group}/types.py"
        message, result = deleteContent(path=path,
            start="# EXAMPLE GRAPHENE ObjectType (Managed by SPRINGLABS_DJANGO)",
            end="# END EXAMPLE GRAPHENE ObjectType (Managed by SPRINGLABS_DJANGO)")
        if result == False:
            errors.append([message, True])
        # Eliminar bandera START de ExampleDjango
        message, result = findNDelete(path=path,
            find="# EXAMPLE GRAPHENE ObjectType (Managed by SPRINGLABS_DJANGO)")
        if result == False:
            errors.append([message, True])
        # Eliminar bandera END de ExampleDjango
        message, result = findNDelete(path=path,
            find="# END EXAMPLE GRAPHENE ObjectType (Managed by SPRINGLABS_DJANGO)")
        if result == False:
            errors.append([message, True])
        
        new_import_schema = f"from .types import {name.capitalize()}Type"
        new_import_api = f"from .{group}.schema import {name.capitalize()}Queries"
        new_query_api = "class Query("

        counter = 0
        index_replace_example = 0
        for index, old_graph in enumerate(old_graphs_version):
            if old_graph != f"{example_django_name}Queries":
                if index == 0:
                    new_query_api = new_query_api+f"{old_graph}"
                else:
                    new_query_api = new_query_api+f", {old_graph}"
            elif old_graph == f"{example_django_name}Queries" and counter != 0:
                if index == 0:
                    new_query_api = new_query_api+f"{old_graph}"
                else:
                    new_query_api = new_query_api+f", {old_graph}"
            else:
                if index == 0:
                    new_query_api = new_query_api+f"{name.capitalize()}Queries"
                else:
                    new_query_api = new_query_api+f", {name.capitalize()}Queries"
                index_replace_example = index
                counter = counter+1
        for index, old_graph in enumerate(old_graphs):
            if old_graph != example_django_name:
                if index == 0:
                    new_import_schema = new_import_schema+f" {old_graph.capitalize()}Type"
                    new_import_api = new_import_api+f" {old_graph.capitalize()}Queries"
                else:
                    new_import_schema = new_import_schema+f", {old_graph}Type"
                    new_import_api = new_import_api+f", {old_graph.capitalize()}Queries"
        index_graph_example_django = False
        try:
            with open(CONFIG_JSON) as file:
                data = json.load(file)
        except:
            message = f"An internal error occurred while updating file {CONFIG_JSON}"
            errors.append([message, True])
        else:
            versions_detail = data['versions_detail']
            # Recorremos todas las versiones
            for index_versions, versions in enumerate(versions_detail):
                # Si la versión del ciclo es en donde vamos a insertar el graph
                if versions['version'] == version:
                    groups_detail = versions["groups_detail"]
                    
                    # Recorremos todos los grupos de esa versión
                    for index_groups, group_detail in enumerate(groups_detail):
                        # Si el grupo del ciclo es en donde vamos a insertar el graph
                        if group == group_detail['group']:
                            graphs_json = group_detail['graph']
                            # Recorremos los graphs en la aplicación y grupo especificados
                            for index_graph, graph_json in enumerate(graphs_json):
                                if graph_json['graph_name'] == example_django_name:
                                    index_graph_example_django = [index_versions,index_groups,index_graph]
            # Si existió graphQL en la lista de graphs de la versión y grupo
            if index_graph_example_django != False:
                # Elimina el graph de Example Django
                index_version = index_graph_example_django[0]
                index_groups = index_graph_example_django[1]
                index_graph = index_graph_example_django[2]
                data['versions_detail'][index_version]['graph_list'][index_replace_example] = f"{name.capitalize()}Queries"
                data['versions_detail'][index_version]['groups_detail'][index_groups]['graph'].pop(index_graph)
            try:
                with open(CONFIG_JSON, "w") as file:
                    file.write(json.dumps(data))
            except:
                message = f"An internal error occurred while updating file {CONFIG_JSON}"
                errors.append([message, True])
    else:
        new_import_schema = find_import_schema + f", {name.capitalize()}Type"
        new_import_api = find_import_api + f", {name.capitalize()}Queries"
        new_query_api = base_new_query_api + f", {name.capitalize()}Queries"
        # Modifica graph_list
        try:
            with open(CONFIG_JSON) as file:
                data = json.load(file)
        except:
            message = f"An internal error occurred while updating file {CONFIG_JSON}"
            errors.append([message, True])
        else:
            data['versions_detail'][index_version_modify]['graph_list'].append(f"{name.capitalize()}Queries")
            try:
                with open(CONFIG_JSON, "w") as file:
                    file.write(json.dumps(data))
            except:
                message = f"An internal error occurred while updating file {CONFIG_JSON}"
                errors.append([message, True])
        
        
    new_query_api = new_query_api + ", graphene.ObjectType):"
    # Escribe nuevo graph en archivo schema.py
    path = f"graphql_api/v{version}/{group}/schema.py"
    message, result = appendContent(path=path,
        content=template_schema_py)
    if result == False:
        errors.append([message, True])
    
    # Modifica imports en schema.py
    message, result = findNReplace(path=path,
        content=new_import_schema,
        find=find_import_schema)
    if result == False:
        errors.append([message, True])

    # Escribe en archivo types.py
    path = f"graphql_api/v{version}/{group}/types.py"
    message, result = appendContent(path=path,
        content=template_types_py)
    if result == False:
        errors.append([message, True])

    # Escribe en archivo api.py
    path = f"graphql_api/v{version}/api.py"
    message, result = findNReplace(path=path,
        content=new_import_api,
        find=find_import_api)
    if result == False:
        errors.append([message, True])
    
    message, result = findNReplace(path=path,
        content=new_query_api,
        find=find_query_api)
    if result == False:
        errors.append([message, True])

    for error in errors:
        if error[1] == True:
            return error[0], False
    return "OK", True

def modifyFilesNewGroup(version, group,project_name):
    """
        Función encargada de modificar archivos para un nuevo grupo en django

        Función encargada de modificar los archivos necesarios para crear un nuevo grupo
        en el proyecto django.

        Parámetros:
            version         [String]    Versión donde se creará el grupo
            group           [String]    Nombre del grupo
            
        Retorno:
            message,result  [Tuple]     Mensaje[String] y result[Boolean]
        Exceptions:
            OSError                     Error al abrir o escribir archivos

    """
    errors = []
    # Crear version en project_name/urls.py
    # Importar public versions de nueva versión
    groupcap=group.capitalize()
    publicpath = f"api/v{version}/public_urls.py"
    privatepath = f"api/v{version}/private_urls.py"
    find = f"# v{version} PATTERNS (Managed by SPRINGLABS_DJANGO)"
    uppergroup=group.upper()
    content = f"""
# START {uppergroup} URLS v{version} (Managed by SPRINGLABS_DJANGO)
{group}_patterns = [
    # {uppergroup} URLS (Managed by SPRINGLABS_DJANGO)
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
# END {uppergroup} URLS v{version} (Managed by SPRINGLABS_DJANGO)

"""
    message, result = appendContent(path=publicpath,
        content=content,
        find=find)
    if result == False:
        errors.append([message, True])

    message, result = appendContent(path=privatepath,
        content=content,
        find=find)
    if result == False:
        errors.append([message, True])

    find="# PATTERNS URLS (Managed by SPRINGLABS_DJANGO)"
    content=f"    url(r'^', include({group}_patterns)),"
    message, result = appendContent(path=privatepath,
        content=content,
        find=find)
    if result == False:
        errors.append([message, True])

    message, result = appendContent(path=publicpath,
        content=content,
        find=find)
    if result == False:
        errors.append([message, True])

    
    find="# Our custom libraries (Managed by SPRINGLABS_DJANGO)"
    content=f"from .{group} import views as {group}_views"
    message, result = appendContent(path=privatepath,
        content=content,
        find=find)
    if result == False:
        errors.append([message, True])

    message, result = appendContent(path=publicpath,
        content=content,
        find=find)
    if result == False:
        errors.append([message, True])

    content=f"from .{group}.schema import ExampleDjango{groupcap}Queries"
    path=f"graphql_api/v{version}/api.py"
    message, result = appendContent(path=path,
        content=content,
        find=find)
    if result == False:
        errors.append([message, True])

    try:
        with open(CONFIG_JSON) as file:
            data = json.load(file)
    except:
        message = F"An internal error occurred while reading file ({CONFIG_JSON})"
        return message, False

    versions_detail=data["versions_detail"]
    version_data=None
    for obj in versions_detail:
        if obj["version"]==version:
            if "graph_list" in obj:
                version_data=obj["graph_list"]
            else:
                version_data=[]

    if version_data!=None:
        content="class Query("
        for obj in version_data:
            content=content+obj+", "
        find=content+"graphene.ObjectType):"

        content=content+f"ExampleDjango{groupcap}Queries, graphene.ObjectType):"
        message, result=findNReplace(path, content, find)
        if result == False:
            errors.append([message, True])

    else:
        message = F"An internal error occurred while reading file ({CONFIG_JSON})"
        return message, False

    for error in errors:
        if error[1] == True:
            return error[0], False
    
    return "OK", True

def createDirectoriesNewGroup(version, group, project_name):
    """
        Función que modifica directorios para agregar nuevo grupo

        Función que modifica los directorios de Django y sus archivos dentro 
        en los que se tiene que agregar el nuevo grupo

        Parámetros:
                version         [String]    Versión donde se creará el grupo
                group           [String]    Nombre del grupo
                project_name    [String]    Nombre del proyecto
                
        Retorno:
                message,result  [Tuple]     Mensaje[String] y result[Boolean]
        Exceptions:
                OSError                     Error al abrir o escribir archivos
    """
    errors = []
    os.chdir(f"api/v{version}")
    os.mkdir(group)

    base_serializers_py = "/core/new_version/api/baseSerializers.py"
    base_views_py = "/core/new_version/api/baseViews.py"
    
    # Obtiene contenido de core/new_version/api/baseSerializers.py
    try:
        template_serializers_py = pkgutil.get_data(PROJECT_PACKAGE, base_serializers_py).decode('utf-8')
    except:
        message = f"An internal error occurred in core/new_version/api/baseSerializers.py"
        errors.append([message, True])

    # Obtiene contenido de core/new_version/api/baseViews.py
    try:
        template_views_py = pkgutil.get_data(PROJECT_PACKAGE, base_views_py).decode('utf-8')
    except:
        message = f"An internal error occurred in core/new_version/api/baseViews.py"
        errors.append([message, True])

    # Escribe contenido base en archivo api/new_version/users/serializers.py
    with open(f"{group}/serializers.py", "w") as file:
        file. write(template_serializers_py)
    
    # Escribe contenido base en archivo api/new_version/users/views.py
    with open(f"{group}/views.py", "w") as file:
        file. write(template_views_py)
    os.chdir("../..")

    os.chdir(f"graphql_api/v{version}")
    os.mkdir(group)
    base_schema_py = "/core/new_group/graph/initial_schema.py"
    base_types_py = "/core/new_group/graph/initial_type.py"
    groupcap=group.capitalize()
    changename=f"ExampleDjango{groupcap}"

    # Obtiene contenido de core/new_version/graphql_api/baseSchema.py
    try:
        template_schema_py = pkgutil.get_data(PROJECT_PACKAGE, base_schema_py).decode('utf-8')
        template_schema_py = template_schema_py.replace("GRAPH_NAME", changename)
    except:
        message = f"An internal error occurred in core/new_version/graphql_api/baseSchema.py"
        errors.append([message, True])

    # Obtiene contenido de core/new_version/graphql_api/baseTypes.py
    try:
        template_types_py = pkgutil.get_data(PROJECT_PACKAGE, base_types_py).decode('utf-8')
        template_types_py = template_types_py.replace("GRAPH_NAME", changename)
    except:
        message = f"An internal error occurred in core/new_version/graphql_api/baseTypes.py"
        errors.append([message, True])

     # Escribe contenido base en archivo graphql_api/new_version/users/schema.py
    with open(f"{group}/schema.py", "w") as file:
        file. write(template_schema_py)
    
    # Escribe contenido base en archivo graphql_api/new_version/users/types.py
    with open(f"{group}/types.py", "w") as file:
        file. write(template_types_py)
    os.chdir("../..")
    for error in errors:
        if error[1] == True:
            return error[0], False

    return "OK", True


import click
import time
import os
import re
from springlabs_django.utils import createVersion, createAPI, createGraph, getVersions, createGroup
from springlabs_django.validators import *
from springlabs_django import __version__ as version
from springlabs_django import CONFIG_JSON, PROJECT_NAME, logging


@click.group(invoke_without_command=False)
@click.version_option(version=version, prog_name=PROJECT_NAME, message="%(prog)s, v%(version)s")
@click.pass_context
def cli(ctx):
	"""Springlabs Django Manager."""
	ctx.invoked_subcommand


@cli.command()
@click.option('-ve', '--version',
	prompt='Insert the version number',
	help='New version for the application',
	callback=validate_version_version)
def create_version(version):
	""" Create a new version for django project """
	message, result = createVersion(version=version,
		project_name=project_name,
		old_versions=versions)
	if result == True:
		message = f"Version '{version}' was created successfully"
		logging.info(message)
		click.secho(message, fg='green')
	else:
		message = "Error: " + message
		click.secho(message, fg='red')

@cli.command()
@click.option('-ve','--version',
	prompt='Insert version',
	help='Version to which the group will be added',
	type=click.Choice(getVersions(), case_sensitive=False)
	)
@click.option('-gr','--group',
	prompt='Insert group name' ,
	help='New group for the application',
	callback=validate_group_group)
def create_group(group,version):
	""" Create a new group for django project """
	message, result = createGroup(version=version,
		group=group,
		project_name=project_name
		)
	if result == True:
		message = f"Group '{group}' was created successfully"
		logging.info(message)
		click.secho(message, fg='green')
	else:
		message = "Error: " + message
		click.secho(message, fg='red')


@cli.command()
@click.option('-ve', '--version',
	prompt='Insert version',
	type=click.Choice(versions, case_sensitive=False),
	help='Version to use',
	callback=validate_api_version)
@click.option('-g', '--group',
	prompt='Insert group',
	type=click.Choice(groups_api, case_sensitive=False),
	help='Group to use')
@click.option('-n', '--name',
	prompt='Insert api name',
	help='API name',
	callback=validate_api_name) 
@click.option('-http', '--http_verb',
	prompt='Insert http verb',
	default="POST",
	type=click.Choice(["POST","PUT","DELETE"], case_sensitive=False),
	show_default=True,
	help='HTTP verb to use in the api',
	callback=validate_api_name_verb)
@click.option('-t', '--type_api',
	prompt='Insert api type',
	default="open",
	type=click.Choice(["model","open"], case_sensitive=False),
	show_default=True,
	help='API type')
@click.option('-u', '--url',
	prompt='Insert api url',
	help='API url',
	callback=validate_api_url)
@click.option('-a', '--availability',
	prompt='Insert api availability',
	default="public",
	type=click.Choice(["public","private"], case_sensitive=False),
	show_default=True,
	help='API availability')
def create_api(version,group,name,http_verb,type_api,url,availability):
	""" Create a new api for django project """
	message, result = createAPI(version=version,
		group=group,
		name=name,
		http_verb=http_verb,
		type_api=type_api,
		url=url,
		availability=availability,
		project_name=project_name)
	if result == True:
		message = f"The api '{name}' was created successfully"
		logging.info(message)
		click.secho(message, fg='green')
	else:
		message = "Error: " + message
		click.secho(message, fg='red')

@cli.command()
@click.option('-ve', '--version',
	prompt='Insert version',
	type=click.Choice(versions, case_sensitive=False),
	help='Version to use',
	callback=validate_api_version)
@click.option('-g', '--group',
	prompt='Insert group',
	type=click.Choice(groups_api, case_sensitive=False),
	help='Group to use')
@click.option('-n', '--name',
	prompt='Insert graph name',
	help='GraphQL name',
	callback=validate_graph_name) 
@click.option('-t', '--type_graph',
	prompt='Insert graph type',
	default="open",
	type=click.Choice(["model","open"], case_sensitive=False),
	show_default=True,
	help='Graph type')
def create_graph(version, group, name,type_graph):
	""" Create a new graph for django project """
	message, result = createGraph(version=version,
		group=group,
		name=name,
		type_graph=type_graph,
		project_name=project_name)
	if result == True:
		message = f"Graph '{name}' was successfully created"
		logging.info(message)
		click.secho(message, fg='green')
	else:
		message = "Error: " + message
		click.secho(message, fg='red')

if __name__ == '__main__':
	cli()



import logging

__version__ = "0.3.5"
DEBUG = True

PROJECT_NAME = "Springlabs Django Manager"
PROJECT_PACKAGE = "springlabs_django"
CONFIG_JSON = "springlabs_django.json"
LOG_NAME = "../springlabs_django.log"

format_log = '[%(asctime)s] [%(levelname)s] :: (%(process)d) :: (%(filename)s - %(funcName)s - %(lineno)d ) :: %(message)s'

logging.basicConfig(filename=LOG_NAME,
    format=format_log,
    level = logging.INFO if DEBUG == False else logging.DEBUG)



# GRAPH_NAME GRAPHENE ObjectType (Managed by SPRINGLABS_DJANGO)
class GRAPH_NAMEType(DjangoObjectType):
    class Meta:
        name = "GRAPH_NAME"
        description = "Manejo de GRAPH_NAME"
        model = UserApp
        fields = ["last_name",]
# END GRAPH_NAME GRAPHENE ObjectType (Managed by SPRINGLABS_DJANGO)


# Django libraries
from django.conf.urls import url, include
from django.conf import settings
from django.conf.urls.static import static
# 3rd party libraries
# Standard/core python libraries
# Our custom libraries
from .private_urls import private_urls
from .public_urls import public_urls
from core.customDocumentation import include_docs_urls
from core.documentation import (
    title_name_version_0,
    description_name_version_0,
    namespace_name_version_0,
    urls_name_version_0,
    title_name_version_0_private,
    description_name_version_0_private,
    namespace_name_version_0_private,
    urls_name_version_0_private,
)

# PUBLIC URLS name_version
urls_name_version_0 = [
    url(r'^name_version/', include(public_urls)),
]
# PRIVATE URLS name_version
urls_name_version_0_private = [
    url(r'^name_version/', include(private_urls)),
]

# DOCUMENTATION URLS name_version
doc_patterns = [
    url(r'^', include_docs_urls(title=title_name_version_0,
        patterns=urls_name_version_0,
        description=description_name_version_0,
        namespace=namespace_name_version_0)),
    url(r'^private', include_docs_urls(title=title_name_version_0_private,
        patterns=urls_name_version_0_private,
        description=description_name_version_0_private,
        namespace=namespace_name_version_0_private)),
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

# GENERAL URLS name_version
urlpatterns = [
    url(r'^', include(doc_patterns)),
    url(r'^', include(public_urls)),
    url(r'^', include(private_urls)),
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



# Django libraries
from django.conf.urls import url, include
from django.conf import settings
from django.conf.urls.static import static
# 3rd party libraries
# Standard/core python libraries
# Our custom libraries (Managed by SPRINGLABS_DJANGO)
from .users import views as users_views

# name_version PATTERNS (Managed by SPRINGLABS_DJANGO)
# START USERS URLS name_version (Managed by SPRINGLABS_DJANGO)
users_patterns = [
    # USERS URLS (Managed by SPRINGLABS_DJANGO)
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
# END USERS URLS name_version (Managed by SPRINGLABS_DJANGO)

# PRIVATE URLS name_version
private_urls = [
    # PATTERNS URLS (Managed by SPRINGLABS_DJANGO)
    url(r'^', include(users_patterns)),
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



# Django libraries
from django.contrib.auth.models import User
# 3rd party libraries
from rest_framework import serializers
# Standard/core python libraries
# Our custom libraries
from models_app.models import *



# Django libraries
# 3rd Party libraries
import graphene
from graphene_django.types import DjangoObjectType
# Standard/core python libraries
# Our custom libraries
from models_app.models import UserApp

# EXAMPLE GRAPHENE ObjectType (Managed by SPRINGLABS_DJANGO)
class GRAPH_NAMEType(graphene.ObjectType):
    id_user = graphene.ID(description="A unique int value identifying User",
        required=False)
    username = graphene.String(description="A unique string value identifying User",
        required=False)
    email = graphene.String(description="A unique string value identifying User's email",
                            required=False)
    first_name = graphene.String(description="A string value identifying User's name",
        required=False)
    last_name = graphene.String(description="A string value identifying User's last name",
                                required=False)

    class Meta:
        name = "GRAPH_NAME"
        description = "Manejo de GRAPH_NAME"
# END EXAMPLE GRAPHENE ObjectType (Managed by SPRINGLABS_DJANGO)


# Django libraries
from django.conf.urls import url, include
from django.conf import settings
from django.conf.urls.static import static
from django.views.decorators.csrf import csrf_exempt
# 3rd party libraries
from graphene_django.views import GraphQLView
# Standard/core python libraries
# Our custom libraries
from .api import schema

# GENERAL URLS name_version
urlpatterns = [
    url(r'^', csrf_exempt(GraphQLView.as_view(
        graphiql=settings.DEBUG, schema=schema))),
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



# START API API_NAMESerializerPut (Created by SPRINGLABS_DJANGO)
class API_NAMESerializerPut(serializers.Serializer):
    field = serializers.CharField(label="Username",
        help_text="A unique string value identifying User",
        required=True,
        max_length=100)

    def validate_field(self, value):
        """
            Breve descripción de método que valida campo field.

            Descripción detallada de método que valida campo field, esta
            descripción detallada puede llevar varias lineas.
            Parámetros:
                param           [Type]         Descripción de param
            Excepciones:
                Excepcion       [Type]         Descripción de excepción
            Retorno:
                retorno        [Type]          Descripción de retorno
        """
        return value

    def validate(self, validated_data):
        """
            Breve descripción de método validate.

            Descripción detallada de método validate, esta
            descripción detallada puede llevar varias lineas.
            Parámetros:
                param           [Type]          Descripción de param
            Excepciones:
                Excepcion       [Type]          Descripción de excepción
            Retorno:
                retorno        [Type]          Descripción de retorno
        """
        return validated_data

    def update(self, instance, validated_data):
        """
            Breve descripción de método update.

            Descripción detallada de método update, esta
            descripción detallada puede llevar varias lineas.
            Parámetros:
                param           [Type]          Descripción de param
            Excepciones:
                Excepcion       [Type]          Descripción de excepción
            Retorno:
                retorno        [Type]          Descripción de retorno
        """
        return validated_data
# END API API_NAMESerializerPut (Created by SPRINGLABS_DJANGO)



# Django libraries
# 3rd Party libraries
import graphene
# Standard/core python libraries
# Our custom libraries (Managed by SPRINGLABS_DJANGO)
from .users.schema import ExampleDjangoUsersQueries

# QUERY EXPOSE TO SCHEMA GRPAHENE


class Query(ExampleDjangoUsersQueries, graphene.ObjectType):
    pass


# SCHEMA
schema = graphene.Schema(query=Query, auto_camelcase=False)



# START API API_NAMESerializerDelete (Created by SPRINGLABS_DJANGO)
class API_NAMESerializerDelete(serializers.Serializer):
    def validate(self, validated_data):
        """
            Breve descripción de método validate.

            Descripción detallada de método validate, esta
            descripción detallada puede llevar varias lineas.
            Parámetros:
                param           [Type]          Descripción de param
            Excepciones:
                Excepcion       [Type]          Descripción de excepción
            Retorno:
                retorno        [Type]          Descripción de retorno
        """
        return validated_data

    def destroy(self, instance, validated_data):
        """
            Breve descripción de método destroy.

            Descripción detallada de método destroy, esta
            descripción detallada puede llevar varias lineas.
            Parámetros:
                param           [Type]          Descripción de param
            Excepciones:
                Excepcion       [Type]          Descripción de excepción
            Retorno:
                retorno        [Type]          Descripción de retorno
        """
        return validated_data
# END API API_NAMESerializerDelete (Created by SPRINGLABS_DJANGO)
